<?php

namespace Idealcheckout\Payment\Controller\Payment;
use Magento\Framework\Controller\ResultFactory;
use Magento\Payment\Helper\Data as PaymentHelper;



$sRootPath = dirname(__FILE__);

while((strlen($sRootPath) > 2) && !is_dir($sRootPath . '/idealcheckout'))
{
	$sRootPath = dirname($sRootPath);
}

if(!is_dir($sRootPath . '/idealcheckout'))
{
	die('Cannot find /idealcheckout/ folder.<br>File: ' . __FILE__ . '<br>Line: ' . __LINE__);
}

require_once($sRootPath . '/idealcheckout/includes/library.php');

 
class Start  extends \Magento\Framework\App\Action\Action
{
	/**
     * @var \Magento\Customer\Model\Session
     */
    protected $customerSession;

    /**
     * @var \Magento\Checkout\Model\Session
     */
    protected $checkoutSession;
	
	/**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $scopeConfig;
	
	/**
     * @var \Magento\Framework\UrlInterface
     */
    protected $urlBuilder;
	
	/**
     * @var \Magento\Quote\Model\Quote
     */
    protected $quote = false;
	
	/** 
	var PaymentHelper 
	*/ 
	protected $_paymentHelper; 

	
	/**
     * @param \Magento\Framework\App\Action\Context $context
     * @param \Magento\Customer\Model\Session $customerSession
     * @param \Magento\Checkout\Model\Session $checkoutSession
	 * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        \Magento\Framework\App\Action\Context $context,
        \Magento\Customer\Model\Session $customerSession,
        \Magento\Checkout\Model\Session $checkoutSession,
		\Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
		PaymentHelper $paymentHelper//,
		//\Psr\Log\LoggerInterface $logger, //log injection
    ) {
		//$this->_logger = $logger;
		//$this->_logger->addDebug('some text or variable');
        $this->customerSession = $customerSession;
        $this->checkoutSession = $checkoutSession;
		$this->scopeConfig = $scopeConfig;
		$this->_paymentHelper = $paymentHelper;
		parent::__construct($context);
    }
	
	
	
    public function execute()
    {		
		$sOrderId = $this->checkoutSession->getLastRealOrderId();
		$oOrder = $this->_objectManager->create('Magento\Sales\Model\Order')->loadByIncrementId($sOrderId);

		// Load database settings
		$aDatabaseSettings = idealcheckout_getDatabaseSettings();
		
		$sPaymentCode = $oOrder->getPayment()->getMethodInstance()->getCode();
		$sMethod = substr($sPaymentCode, 14);		
		
		$sStoreCode = idealcheckout_getStoreCode();
		$sGatewayCode = $sMethod;
		$sLanguageCode = 'nl'; // nl, de, en
		$sCountryCode = '';
		$sCurrencyCode = $oOrder->getBaseCurrencyCode();

		if(strcasecmp($sCurrencyCode, 'GBP') === 0)
		{
			// $sLanguageCode = 'EN';
			$sCountryCode = 'UK';
		}		
		
		$sOrderId = $sOrderId;
		$sOrderCode = idealcheckout_getRandomCode(32);
		$aOrderParams = array();
		$sTransactionId = idealcheckout_getRandomCode(32);
		$sTransactionCode = idealcheckout_getRandomCode(32);
		$fTransactionAmount = $oOrder->getBaseGrandTotal();
		$sTransactionDescription = idealcheckout_getTranslation($sLanguageCode, 'idealcheckout', 'Webshop order #{0}', array($sOrderId));

		
		$aQuery = array('order_id' => $sOrderId, 'order_code' => $sOrderCode);
		$sReturnUrl = $this->_url->getUrl('idealcheckout/payment/returnpayment', $aQuery); 
		$sNotifyUrl = $this->_url->getUrl('idealcheckout/payment/notify', $aQuery); 


		$sTransactionPaymentUrl = $sReturnUrl;
		$sTransactionSuccessUrl = $sReturnUrl;
		$sTransactionPendingUrl = $sReturnUrl;
		$sTransactionFailureUrl = $sReturnUrl;

		// Store ORDER information
		$aOrderParams['order'] = array(
			'id' => $sOrderId
		);

		$aOrderParams['customer'] = array();
	
		// Payment data
		list($sStreetName, $sStreetNumber, $sExtension) = idealcheckout_mb_splitAddress($oOrder->getBillingAddress()->getStreetLine(1));

		$aOrderParams['customer']['payment_company'] = $oOrder->getBillingAddress()->getCompany();
		$aOrderParams['customer']['payment_name'] = trim($oOrder->getBillingAddress()->getFirstname() . ' ' . $oOrder->getBillingAddress()->getLastname());
		$aOrderParams['customer']['payment_first_name'] = $oOrder->getBillingAddress()->getFirstname();
		$aOrderParams['customer']['payment_last_name'] = $oOrder->getBillingAddress()->getLastname();
		$aOrderParams['customer']['payment_gender'] = '';
		$aOrderParams['customer']['payment_date_of_birth'] = '';
		$aOrderParams['customer']['payment_phone'] = $oOrder->getBillingAddress()->getTelephone();
		$aOrderParams['customer']['payment_email'] = $oOrder->getBillingAddress()->getEmail();
		$aOrderParams['customer']['payment_address'] = $oOrder->getBillingAddress()->getStreetLine(1);
		$aOrderParams['customer']['payment_street_name'] = $sStreetName;
		$aOrderParams['customer']['payment_street_number'] = $sStreetNumber;
		$aOrderParams['customer']['payment_street_number_extension'] = $sExtension;		
		$aOrderParams['customer']['payment_zipcode'] = $oOrder->getBillingAddress()->getPostcode();
		$aOrderParams['customer']['payment_city'] = $oOrder->getBillingAddress()->getCity();
		$aOrderParams['customer']['payment_country_code'] = $oOrder->getBillingAddress()->getCountryId();
		$aOrderParams['customer']['payment_country_name'] = ((strcasecmp($oOrder->getBillingAddress()->getCountryId(), 'BE') === 0) ? 'Belgie' : 'Nederland');

		$oShipmentAddress = empty($oOrder->getShippingAddress()) ? $oOrder->getBillingAddress() : $oOrder->getShippingAddress();
		
		// Shipment data
		list($sStreetName, $sStreetNumber, $sExtension) = idealcheckout_mb_splitAddress($oShipmentAddress->getStreetLine(1));

		$aOrderParams['customer']['shipment_company'] = $oShipmentAddress->getCompany();
		$aOrderParams['customer']['shipment_name'] = trim($oShipmentAddress->getFirstname() . ' ' . $oShipmentAddress->getLastname());
		$aOrderParams['customer']['shipment_first_name'] = $oShipmentAddress->getFirstname();
		$aOrderParams['customer']['shipment_last_name'] = $oShipmentAddress->getLastname();
		$aOrderParams['customer']['shipment_gender'] = '';
		$aOrderParams['customer']['shipment_date_of_birth'] = '';
		$aOrderParams['customer']['shipment_phone'] = $oShipmentAddress->getTelephone();
		$aOrderParams['customer']['shipment_email'] = $oShipmentAddress->getEmail();
		$aOrderParams['customer']['shipment_address'] = $oShipmentAddress->getStreetLine(1);
		$aOrderParams['customer']['shipment_street_name'] = $sStreetName;
		$aOrderParams['customer']['shipment_street_number'] = $sStreetNumber;
		$aOrderParams['customer']['shipment_street_number_extension'] = $sExtension;
		$aOrderParams['customer']['shipment_zipcode'] = $oShipmentAddress->getPostcode();
		$aOrderParams['customer']['shipment_city'] = $oShipmentAddress->getCity();
		$aOrderParams['customer']['shipment_country_code'] = $oShipmentAddress->getCountryId();
		$aOrderParams['customer']['shipment_country_name'] = ((strcasecmp($oShipmentAddress->getCountryId(), 'BE') === 0) ? 'Belgie' : 'Nederland');

		// Product data
		$aOrderParams['products'] = array();

		foreach($oOrder->getAllVisibleItems() as $k => $oCartItem)
		{
			if(!empty($oCartItem->getName()))
			{
				$sDescription = $oCartItem->getName();
			}
			else
			{
				$sDescription = $oCartItem->getSku();
			}

			$iQuantity = (int)($oCartItem->getQtyOrdered() ? $oCartItem->getQtyOrdered() : $oCartItem->getQty());
			
			$iTaxPercentage = round($oCartItem->getTaxPercent() * 100, 0);
			$fTaxAmount = round($oCartItem->getTaxAmount() * 100, 0);
			$fPriceExcl = round($oCartItem->getPrice() * 100, 0);
			$fPriceIncl = round($oCartItem->getRowTotalInclTax() * 100, 0);
			
			$aOrderParams['products'][] = array('code' => $oCartItem->getSku(), 'description' => $sDescription, 'quantity' => $iQuantity, 'price_incl' => $fPriceIncl, 'price_excl' => $fPriceExcl, 'vat' => $iTaxPercentage);
		}
		
		
		// Shipping costs?		
		if($oOrder->getShippingAmount() != 0)
		{
			$fBaseShipmentAmount = floatval($oOrder->getShippingAmount());
			$fBaseShipmentTaxAmount = floatval($oOrder->getShippingTaxAmount());
			$fBaseShipmentTax = round(($fBaseShipmentTaxAmount / $fBaseShipmentAmount) * 100);

			$aOrderParams['products'][] = array('code' => 'SHIPPING', 'description' => 'Verzendkosten', 'quantity' => '1', 'price_incl' => ($fBaseShipmentAmount + $fBaseShipmentTaxAmount), 'price_excl' => $fBaseShipmentAmount, 'vat' => $fBaseShipmentTax);
		}
		
		// Discount?
		if($oOrder->getDiscountAmount() != 0)
		{
			$fBaseDiscountAmount = floatval($oOrder->getDiscountAmount());
			$fBaseDiscountTaxAmount = 0;
			$fBaseDiscountTax = 0;

			$aOrderParams['products'][] = array('code' => 'DISCOUNT', 'description' => 'Korting en overige kosten', 'quantity' => '1', 'price_incl' => ($fBaseDiscountAmount + $fBaseDiscountTaxAmount), 'price_excl' => $fBaseDiscountAmount, 'vat' => $fBaseDiscountTax);
		}	


		// Payment costs?
		// if(isset($aOrder['base_payment_amount']) && ($aOrder['base_payment_amount'] > 0))
		// {
		//	$fBasePaymentAmount = floatval($aOrder['base_payment_amount']);
		//	$fBasePaymentTaxAmount = 0;
		//	$fBasePaymentTax = 0;
		//	$aOrderParams['products'][] = array('code' => 'PAYMENT', 'description' => 'Betaalkosten', 'quantity' => '1', 'price_incl' => ($fBasePaymentAmount + $fBasePaymentTaxAmount), 'price_excl' => $fBasePaymentAmount, 'vat' => $fBasePaymentTax);
		// }

		// Insert data into idealcheckout-table
		$sql = "INSERT INTO `" . $aDatabaseSettings['table'] . "` SET 
`id` = NULL, 
`order_id` = '" . idealcheckout_escapeSql($sOrderId) . "', 
`order_code` = '" . idealcheckout_escapeSql($sOrderCode) . "', 
`order_params` = '" . idealcheckout_escapeSql(idealcheckout_serialize($aOrderParams)) . "', 
`store_code` = " . (empty($sStoreCode) ? "NULL" : "'" . idealcheckout_escapeSql($sStoreCode) . "'") . ", 
`gateway_code` = '" . idealcheckout_escapeSql($sGatewayCode) . "', 
`language_code` = " . (empty($sLanguageCode) ? "NULL" : "'" . idealcheckout_escapeSql($sLanguageCode) . "'") . ", 
`country_code` = " . (empty($sCountryCode) ? "NULL" : "'" . idealcheckout_escapeSql($sCountryCode) . "'") . ", 
`currency_code` = '" . idealcheckout_escapeSql($sCurrencyCode) . "', 
`transaction_id` = '" . idealcheckout_escapeSql($sTransactionId) . "', 
`transaction_code` = '" . idealcheckout_escapeSql($sTransactionCode) . "', 
`transaction_params` = NULL, 
`transaction_date` = '" . idealcheckout_escapeSql(time()) . "', 
`transaction_amount` = '" . idealcheckout_escapeSql($fTransactionAmount) . "', 
`transaction_description` = '" . idealcheckout_escapeSql($sTransactionDescription) . "', 
`transaction_status` = NULL, 
`transaction_url` = NULL, 
`transaction_payment_url` = '" . idealcheckout_escapeSql($sTransactionPaymentUrl) . "', 
`transaction_success_url` = '" . idealcheckout_escapeSql($sTransactionSuccessUrl) . "', 
`transaction_pending_url` = '" . idealcheckout_escapeSql($sTransactionPendingUrl) . "', 
`transaction_failure_url` = '" . idealcheckout_escapeSql($sTransactionFailureUrl) . "', 
`transaction_notify_url` = '" . idealcheckout_escapeSql($sNotifyUrl) . "', 
`transaction_log` = NULL;";


		// Add record to transaction table
		idealcheckout_database_query($sql);
	
	
	

		$objectManager = \Magento\Framework\App\ObjectManager::getInstance();
		$storeManager = $objectManager->get('\Magento\Store\Model\StoreManagerInterface');
		$sUrl = $storeManager->getStore()->getBaseUrl();		
		
		$this->getResponse()->setRedirect($sUrl . 'idealcheckout/setup.php?order_id=' . $sOrderId . '&order_code=' . $sOrderCode);
		
    }
	
	
	/**
     * Returns action name which requires redirect
     * @return string
     */
    public function getRedirectActionName()
    {
        return 'start';
    }
}